@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 #include <stdlib.h>
 #include <string.h>
@@ -388,14 +388,20 @@ void bctbx_DHMCreatePublic(bctbx_DHMContext_t *context, int (*rngFunction)(void
 /* compute secret - the ->peer field of context must have been set before calling this function */
 void bctbx_DHMComputeSecret(bctbx_DHMContext_t *context, int (*rngFunction)(void *, uint8_t *, size_t), void *rngContext) {
 	size_t keyLength;
-
+	uint8_t sharedSecretBuffer[384]; /* longest shared secret available in these mode */
+	
 	/* import the peer public value G^Y mod P in the polar ssl context */
 	dhm_read_public((dhm_context *)(context->cryptoModuleData), context->peer, context->primeLength);
 
 	/* compute the secret key */
 	keyLength = context->primeLength; /* undocumented but this value seems to be in/out, so we must set it to the expected key length */
-	context->key = (uint8_t *)malloc(keyLength*sizeof(uint8_t)); /* allocate key buffer */
-	dhm_calc_secret((dhm_context *)(context->cryptoModuleData), context->key, &keyLength);
+	context->key = (uint8_t *)malloc(keyLength*sizeof(uint8_t)); /* allocate and reset the key buffer */
+	memset(context->key,0, keyLength);
+	
+	dhm_calc_secret((dhm_context *)(context->cryptoModuleData), sharedSecretBuffer, &keyLength);
+	/* now copy the resulting secret in the correct place in buffer(result may actually miss some front zero bytes, real length of output is now in keyLength but we want primeLength bytes) */
+	memcpy(context->key+(context->primeLength-keyLength), sharedSecretBuffer, keyLength);
+	memset(sharedSecretBuffer, 0, 384); /* purge secret from temporary buffer */
 }
 
 /* clean DHM context */
@@ -424,6 +430,7 @@ void bctbx_DestroyDHMContext(bctbx_DHMContext_t *context) {
 /** context **/
 struct bctbx_ssl_context_struct {
 	ssl_context ssl_ctx;
+	char *cn;
 	int(*callback_cli_cert_function)(void *, bctbx_ssl_context_t *, unsigned char *, size_t); /**< pointer to the callback called to update client certificate during handshake
 													callback params are user_data, ssl_context, certificate distinguished name, name length */
 	void *callback_cli_cert_data; /**< data passed to the client cert callback */
@@ -444,6 +451,7 @@ bctbx_ssl_context_t *bctbx_ssl_context_new(void) {
 }
 
 void bctbx_ssl_context_free(bctbx_ssl_context_t *ssl_ctx) {
+	if (ssl_ctx->cn) bctbx_free(ssl_ctx->cn);
 	ssl_free(&(ssl_ctx->ssl_ctx));
 	bctbx_free(ssl_ctx);
 }
@@ -557,6 +565,22 @@ const bctbx_x509_certificate_t *bctbx_ssl_get_peer_certificate(bctbx_ssl_context
 	return (const bctbx_x509_certificate_t *)ssl_get_peer_cert(&(ssl_ctx->ssl_ctx));
 }
 
+const char *bctbx_ssl_get_ciphersuite(bctbx_ssl_context_t *ssl_ctx){
+	return ssl_get_ciphersuite(&(ssl_ctx->ssl_ctx));
+}
+
+const char *bctbx_ssl_get_version(bctbx_ssl_context_t *ssl_ctx){
+	return ssl_get_version(&(ssl_ctx->ssl_ctx));
+}
+
+int32_t bctbx_ssl_set_hostname(bctbx_ssl_context_t *ssl_ctx, const char *hostname){
+	if (ssl_ctx->cn) bctbx_free(ssl_ctx->cn);
+	if (hostname) ssl_ctx->cn = bctbx_strdup(hostname);
+	else ssl_ctx->cn = NULL;
+	ssl_ctx->ssl_ctx.peer_cn = ssl_ctx->cn;
+	return 0;
+}
+
 /** dummmy DTLS SRTP functions **/
 uint8_t bctbx_dtls_srtp_supported(void) {
 	return 0;
@@ -582,7 +606,6 @@ struct bctbx_ssl_config_struct {
 	int(*callback_verify_function)(void *, x509_cert *, int, int *); /**< pointer to the verify callback function */
 	void *callback_verify_data; /**< data passed to the verify callback */
 	x509_cert *ca_chain; /**< trusted CA chain */
-	char *peer_cn; /**< expected peer Common name */
 	x509_cert *own_cert;
 	rsa_context *own_cert_pk;
 	int(*callback_cli_cert_function)(void *, bctbx_ssl_context_t *, unsigned char *, size_t); /**< pointer to the callback called to update client certificate during handshake
@@ -604,7 +627,6 @@ bctbx_ssl_config_t *bctbx_ssl_config_new(void) {
 	ssl_config->callback_cli_cert_function = NULL;
 	ssl_config->callback_cli_cert_data = NULL;
 	ssl_config->ca_chain = NULL;
-	ssl_config->peer_cn = NULL;
 	ssl_config->own_cert = NULL;
 	ssl_config->own_cert_pk = NULL;
 	return ssl_config;
@@ -721,10 +743,9 @@ int32_t bctbx_ssl_config_set_callback_cli_cert(bctbx_ssl_config_t *ssl_config, i
 	return BCTBX_ERROR_INVALID_SSL_CONFIG;
 }
 
-int32_t bctbx_ssl_config_set_ca_chain(bctbx_ssl_config_t *ssl_config, bctbx_x509_certificate_t *ca_chain, char *peer_cn) {
+int32_t bctbx_ssl_config_set_ca_chain(bctbx_ssl_config_t *ssl_config, bctbx_x509_certificate_t *ca_chain) {
 	if (ssl_config != NULL) {
 		ssl_config->ca_chain = (x509_cert *)ca_chain;
-		ssl_config->peer_cn = peer_cn;
 	}
 	return BCTBX_ERROR_INVALID_SSL_CONFIG;
 }
@@ -777,7 +798,7 @@ int32_t bctbx_ssl_context_setup(bctbx_ssl_context_t *ssl_ctx, bctbx_ssl_config_t
 	}
 
 	if (ssl_config->ca_chain != NULL) {
-		ssl_set_ca_chain(&(ssl_ctx->ssl_ctx), ssl_config->ca_chain, NULL, ssl_config->peer_cn);
+		ssl_set_ca_chain(&(ssl_ctx->ssl_ctx), ssl_config->ca_chain, NULL, ssl_ctx->cn);
 	}
 
 	if (ssl_config->own_cert != NULL && ssl_config->own_cert_pk != NULL) {
