@@ -30,11 +30,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <unistd.h>
 #endif
 
-#ifndef IN_CUNIT_SOURCES
-#include <CUnit/Basic.h>
-#include <CUnit/Automated.h>
-#include <CUnit/MyMem.h>
-#include <CUnit/Util.h>
+#ifndef IN_BCUNIT_SOURCES
+#include <BCUnit/Basic.h>
+#include <BCUnit/Automated.h>
+#include <BCUnit/MyMem.h>
+#include <BCUnit/Util.h>
 #else
 #include "Basic.h"
 #include "Automated.h"
@@ -65,7 +65,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define F_OK 00 /* Visual Studio does not define F_OK */
 #endif
 
-#ifdef HAVE_CUNIT_CUCURSES_H
+#ifdef HAVE_BCUNIT_CUCURSES_H
 #define HAVE_CU_CURSES 1
 #endif
 
@@ -83,17 +83,18 @@ static test_suite_t **test_suite = NULL;
 static int nb_test_suites = 0;
 
 #ifdef HAVE_CU_CURSES
-#include "CUnit/CUCurses.h"
+#include "BCUnit/CUCurses.h"
 static unsigned char curses = 0;
 #endif
 
-char* xml_file = "CUnitAutomated-Results.xml";
+char* xml_file = "BCUnitAutomated-Results.xml";
 int   xml_enabled = 0;
 char * suite_name = NULL;
 char * test_name = NULL;
 char * tag_name = NULL;
 char * expected_res = NULL;
-static long max_vm_kb = 0;
+static size_t max_vm_kb = 0;
+int run_skipped_tests = 0;
 
 static void (*tester_printf_va)(int level, const char *format, va_list args)=NULL;
 
@@ -113,7 +114,7 @@ int bc_tester_run_suite(test_suite_t *suite, const char *tag_name) {
 	CU_pSuite pSuite;
 
 	if (tag_name != NULL) {
-		int j;
+		size_t j;
 		int nb_tests_for_tag = 0;
 		for (i = 0; i < suite->nb_tests; i++) {
 			for (j = 0; j < (sizeof(suite->tests[i].tags) / sizeof(suite->tests[i].tags[0])); j++) {
@@ -123,7 +124,8 @@ int bc_tester_run_suite(test_suite_t *suite, const char *tag_name) {
 			}
 		}
 		if (nb_tests_for_tag > 0) {
-			pSuite = CU_add_suite(suite->name, suite->before_all, suite->after_all);
+			pSuite = CU_add_suite_with_setup_and_teardown(suite->name, suite->before_all, suite->after_all,
+												suite->before_each, suite->after_each);
 			for (i = 0; i < suite->nb_tests; i++) {
 				for (j = 0; j < (sizeof(suite->tests[i].tags) / sizeof(suite->tests[i].tags[0])); j++) {
 					if ((suite->tests[i].tags[j] != NULL) && (strcasecmp(tag_name, suite->tests[i].tags[j]) == 0)) {
@@ -135,10 +137,20 @@ int bc_tester_run_suite(test_suite_t *suite, const char *tag_name) {
 			}
 		}
 	} else {
-		pSuite = CU_add_suite(suite->name, suite->before_all, suite->after_all);
+		pSuite = CU_add_suite_with_setup_and_teardown(suite->name, suite->before_all, suite->after_all,
+												suite->before_each, suite->after_each);
 		for (i = 0; i < suite->nb_tests; i++) {
-			if (NULL == CU_add_test(pSuite, suite->tests[i].name, suite->tests[i].func)) {
-				return CU_get_error();
+			size_t j;
+			bool_t skip = FALSE;
+			for (j = 0; j < (sizeof(suite->tests[i].tags) / sizeof(suite->tests[i].tags[0])); j++) {
+				if ((suite->tests[i].tags[j] != NULL) && (strcasecmp("Skip", suite->tests[i].tags[j]) == 0) && (run_skipped_tests == 0)) {
+					skip = TRUE;
+				}
+			}
+			if (!skip) {
+				if (NULL == CU_add_test(pSuite, suite->tests[i].name, suite->tests[i].func)) {
+					return CU_get_error();
+				}
 			}
 		}
 	}
@@ -181,10 +193,23 @@ int bc_tester_nb_suites(void) {
 }
 
 const char * bc_tester_test_name(const char *suite_name, int test_index) {
+	test_suite_t *suite = NULL;
+	size_t j = 0;
+	bool_t skip = FALSE;
+	
 	int suite_index = bc_tester_suite_index(suite_name);
 	if ((suite_index < 0) || (suite_index >= nb_test_suites)) return NULL;
-	if (test_index >= test_suite[suite_index]->nb_tests) return NULL;
-	return test_suite[suite_index]->tests[test_index].name;
+	suite = test_suite[suite_index];
+	if (test_index >= suite->nb_tests) return NULL;
+	
+	for (j = 0; j < (sizeof(suite->tests[test_index].tags) / sizeof(suite->tests[test_index].tags[0])); j++) {
+		if ((suite->tests[test_index].tags[j] != NULL) && (strcasecmp("Skip", suite->tests[test_index].tags[j]) == 0) && (run_skipped_tests == 0)) {
+			skip = TRUE;
+		}
+	}
+	if (skip) return NULL;
+	
+	return suite->tests[test_index].name;
 }
 
 int bc_tester_nb_tests(const char *suite_name) {
@@ -204,7 +229,9 @@ void bc_tester_list_tests(const char *suite_name) {
 	int j;
 	for( j = 0; j < bc_tester_nb_tests(suite_name); j++) {
 		const char *test_name = bc_tester_test_name(suite_name, j);
-		bc_tester_printf(bc_printf_verbosity_info, "%s", test_name);
+		if (test_name) {
+			bc_tester_printf(bc_printf_verbosity_info, "%s", test_name);
+		}
 	}
 }
 
@@ -238,20 +265,15 @@ static void suite_complete_message_handler(const CU_pSuite pSuite, const CU_pFai
 
 static uint64_t test_start_time = 0;
 static void test_start_message_handler(const CU_pTest pTest, const CU_pSuite pSuite) {
-	int suite_index = bc_tester_suite_index(pSuite->pName);
-	if (test_suite[suite_index]->before_each) {
-		test_suite[suite_index]->before_each();
-	}
 	bc_tester_printf(bc_printf_verbosity_info,"Suite [%s] Test [%s] started", pSuite->pName,pTest->pName);
 	test_start_time = bctbx_get_cur_time_ms();
 	bc_current_test_name = pTest->pName;
 }
 
-/*derivated from cunit*/
+/*derivated from bcunit*/
 static void test_complete_message_handler(const CU_pTest pTest, const CU_pSuite pSuite,
 										  const CU_pFailureRecord pFailureList) {
 	int i;
-	int suite_index = bc_tester_suite_index(pSuite->pName);
 	CU_pFailureRecord pFailure = pFailureList;
 	char *buffer = NULL;
 	char* result = bc_sprintf("Suite [%s] Test [%s] %s in %.3f secs", pSuite->pName, pTest->pName,
@@ -273,13 +295,6 @@ static void test_complete_message_handler(const CU_pTest pTest, const CU_pSuite
 	bc_tester_printf(bc_printf_verbosity_info,"%s", result);
 	free(result);
 
-	if (test_suite[suite_index]->after_each) {
-		int err = test_suite[suite_index]->after_each();
-		//if test passed but not after_each, count it as failure
-		if (err && !pFailure) {
-			CU_get_run_summary()->nTestsFailed++;
-		}
-	}
 	//insert empty line
 	bc_tester_printf(bc_printf_verbosity_info,"");
 
@@ -295,7 +310,7 @@ static void test_complete_message_handler(const CU_pTest pTest, const CU_pSuite
 	 */
 	if (max_vm_kb) {
 		struct mallinfo minfo = mallinfo();
-		if (minfo.uordblks > max_vm_kb * 1024) {
+		if ((size_t)minfo.uordblks > max_vm_kb * 1024) {
 			bc_tester_printf(
 				bc_printf_verbosity_error,
 				"The program exceeded the maximum amount of memory allocatable (%i bytes), aborting now.\n",
@@ -310,7 +325,7 @@ static void test_complete_message_handler(const CU_pTest pTest, const CU_pSuite
 int bc_tester_run_tests(const char *suite_name, const char *test_name, const char *tag_name) {
 	int i;
 
-	/* initialize the CUnit test registry */
+	/* initialize the BCUnit test registry */
 	if (CUE_SUCCESS != CU_initialize_registry())
 		return CU_get_error();
 
@@ -371,13 +386,13 @@ int bc_tester_run_tests(const char *suite_name, const char *test_name, const cha
 		{
 #ifdef HAVE_CU_CURSES
 			if (curses) {
-			/* Run tests using the CUnit curses interface */
+			/* Run tests using the BCUnit curses interface */
 				CU_curses_run_tests();
 			}
 			else
 #endif
 			{
-				/* Run all tests using the CUnit Basic interface */
+				/* Run all tests using the BCUnit Basic interface */
 				CU_run_all_tests();
 			}
 		}
@@ -391,7 +406,7 @@ int bc_tester_run_tests(const char *suite_name, const char *test_name, const cha
 
 }
 
-#if !defined(BC_TESTER_WINDOWS_PHONE) && !defined(BC_TESTER_WINDOWS_UNIVERSAL) && !defined(__QNX__) && !defined(ANDROID) && !defined(IOS)
+#if !defined(BC_TESTER_WINDOWS_PHONE) && !defined(BC_TESTER_WINDOWS_UNIVERSAL) && !defined(__QNX__) && !defined(__ANDROID__) && !defined(IOS)
 static int file_exists(const char* root_path) {
 	char * res_path = bc_sprintf("%s/%s", root_path, expected_res);
 	int err = bctbx_file_exist(res_path);
@@ -419,10 +434,14 @@ static void detect_res_prefix(const char* prog) {
 			ptr = strrchr(prog, '\\');
 		}
 		if (ptr != NULL) {
+#ifdef BC_TESTER_WINDOWS_DESKTOP
+			char *exe = strstr(prog, ".exe");
+			if (exe != NULL) exe[0] = '\0';
+#endif
 			progname = strdup(ptr + 1);
 		}
 	}
-#if !defined(BC_TESTER_WINDOWS_PHONE) && !defined(BC_TESTER_WINDOWS_UNIVERSAL) && !defined(__QNX__) && !defined(ANDROID) && !defined(IOS)
+#if !defined(BC_TESTER_WINDOWS_PHONE) && !defined(BC_TESTER_WINDOWS_UNIVERSAL) && !defined(__QNX__) && !defined(__ANDROID__) && !defined(IOS)
 	{
 		char* prefix = NULL;
 		char *installed_resources_path = NULL;
@@ -509,9 +528,9 @@ void bc_tester_init(void (*ftester_printf)(int level, const char *format, va_lis
 	}
 }
 
-void bc_tester_set_max_vm(long amax_vm_kb) {
+void bc_tester_set_max_vm(size_t amax_vm_kb) {
 #ifdef __linux
-	max_vm_kb = amax_vm_kb;
+	max_vm_kb = (size_t)amax_vm_kb;
 	bc_tester_printf(bc_printf_verbosity_info, "Maximum virtual memory space set to %li kilo bytes", max_vm_kb);
 #else
 	bc_tester_printf(bc_printf_verbosity_error, "Maximum virtual memory space setting is only implemented on Linux.");
@@ -529,6 +548,7 @@ void bc_tester_helper(const char *name, const char* additionnal_helper) {
 					 "\t\t\t--suite <suite name>\n"
 					 "\t\t\t--test <test name>\n"
 					 "\t\t\t--tag <tag name> (execute all tests with the given tag)\n"
+					 "\t\t\t--all (execute all tests, even the ones with the Skip flag)\n"
 					 "\t\t\t--resource-dir <folder path> (directory where tester resource are located)\n"
 					 "\t\t\t--writable-dir <folder path> (directory where temporary files should be created)\n"
 					 "\t\t\t--xml\n"
@@ -555,6 +575,8 @@ int bc_tester_parse_args(int argc, char **argv, int argid)
 	} else if (strcmp(argv[i], "--tag") == 0) {
 		CHECK_ARG("--tag", ++i, argc);
 		tag_name = argv[i];
+	} else if (strcmp(argv[i], "--all") == 0) {
+		run_skipped_tests = 1;
 	} else if (strcmp(argv[i],"--list-suites")==0){
 		bc_tester_list_suites();
 		return 0;
