@@ -18,10 +18,13 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "bctoolbox/logging.h"
 #include "bctoolbox/port.h"
+#include "bctoolbox/vconnect.h"
 #include "bctoolbox/list.h"
 #include "utils.h"
 
@@ -104,6 +107,37 @@ char * bctbx_strdup(const char *tmp){
 	return ret;
 }
 
+char * bctbx_dirname(const char *path) {
+	char *ptr;
+	char *dname = bctbx_strdup(path);
+	bool_t found = FALSE;
+
+	ptr = strrchr(path, '/');
+	if (ptr != NULL) {
+		dname[ptr - path] = '\0';
+		found = TRUE;
+	} else {
+		ptr = strrchr(path, '\\');
+		if (ptr != NULL) {
+			dname[ptr - path] = '\0';
+			found = TRUE;
+		}
+	}
+
+	if (found == FALSE) {
+		bctbx_free(dname);
+		return NULL;
+	}
+	return dname;
+}
+
+char * bctbx_basename(const char *path) {
+	char *ptr = strrchr(path, '/');
+	if (ptr == NULL) ptr = strrchr(path, '\\');
+	if (ptr == NULL) return NULL;
+	return bctbx_strdup(ptr + 1);
+}
+
 /*
  * this method is an utility method that calls fnctl() on UNIX or
  * ioctlsocket on Win32.
@@ -119,18 +153,6 @@ int bctbx_socket_set_non_blocking(bctbx_socket_t sock){
 }
 
 
-/*
- * this method is an utility method that calls close() on UNIX or
- * closesocket on Win32.
- * int retrun the result of the system method
- */
-int bctbx_socket_close(bctbx_socket_t sock){
-#if	!defined(_WIN32) && !defined(_WIN32_WCE)
-	return close (sock);
-#else
-	return closesocket(sock);
-#endif
-}
 
 int bctbx_file_exist(const char *pathname) {
 	return access(pathname,F_OK);
@@ -365,24 +387,8 @@ int bctbx_gettimeofday (struct timeval *tv, void* tz)
 
 const char *__bctbx_getWinSocketError(int error)
 {
-	static char buf[80];
-
-	switch (error)
-	{
-		case WSANOTINITIALISED: return "Windows sockets not initialized : call WSAStartup";
-		case WSAEADDRINUSE:		return "Local Address already in use";
-		case WSAEADDRNOTAVAIL:	return "The specified address is not a valid address for this machine";
-		case WSAEINVAL:			return "The socket is already bound to an address.";
-		case WSAENOBUFS:		return "Not enough buffers available, too many connections.";
-		case WSAENOTSOCK:		return "The descriptor is not a socket.";
-		case WSAECONNRESET:		return "Connection reset by peer";
-
-		default :
-			sprintf(buf, "Error code : %d", error);
-			return buf;
-		break;
-	}
-
+	static char buf[256];
+	FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, error, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), buf, sizeof(buf), NULL);
 	return buf;
 }
 
@@ -687,7 +693,7 @@ void bctbx_shm_close(void *mem){
 			CloseHandle(i->h);
 			UnmapViewOfFile(mem);
 			bctbx_free(i);
-			maplist=bctbx_list_remove_link(maplist,elem);
+			maplist=bctbx_list_erase_link(maplist,elem);
 			return;
 		}
 	}
@@ -743,6 +749,10 @@ void _bctbx_get_cur_time(bctoolboxTimeSpec *ret, bool_t realtime){
 #endif
 }
 
+void bctbx_get_utc_cur_time(bctoolboxTimeSpec *ret){
+	_bctbx_get_cur_time(ret, TRUE);
+}
+
 void bctbx_get_cur_time(bctoolboxTimeSpec *ret){
 	_bctbx_get_cur_time(ret, FALSE);
 }
@@ -750,7 +760,7 @@ void bctbx_get_cur_time(bctoolboxTimeSpec *ret){
 
 uint64_t bctbx_get_cur_time_ms(void) {
 	bctoolboxTimeSpec ts;
-	bctbx_get_cur_time(&ts);
+	_bctbx_get_cur_time(&ts, TRUE);
 	return (ts.tv_sec * 1000LL) + ((ts.tv_nsec + 500000LL) / 1000000LL);
 }
 
@@ -804,6 +814,109 @@ void bctbx_sleep_until(const bctoolboxTimeSpec *ts){
 #endif
 }
 
+/**
+ * @brief Add given amount of seconds to a timeSpec structure
+ *
+ * @param[in/out]	ts	The timeSpec structure used as input, modified in output by increnting it according to second argument
+ * @param[in]		lap	In seconds, number of seconds to modify the given timeSpec, can be negative(which may set the original timeSpec to 0)
+ */
+void bctbx_timespec_add(bctoolboxTimeSpec *ts, const int64_t lap) {
+	if (lap<0 && -lap > ts->tv_sec) {
+		ts->tv_sec = 0;
+		ts->tv_nsec = 0;
+	} else {
+		ts->tv_sec += lap;
+	}
+}
+
+/**
+ * @brief Compares two TimeSpec s1 and s2.
+ *
+ * @param[in]	s1	First time spec
+ * @param[in]	s2	Second time spec
+ *
+ * @return a negative value if s1 is earlier than s2, 0 if they are equal, a positive value if s1 is later than s2
+ */
+int bctbx_timespec_compare(const bctoolboxTimeSpec *s1, const bctoolboxTimeSpec *s2){
+	int64_t secdiff = s1->tv_sec - s2->tv_sec;
+	if (secdiff == 0){
+		int64_t nsec_diff = s1->tv_nsec - s2->tv_nsec;
+		if (nsec_diff < 0){
+			return -1;
+		}else if (nsec_diff > 0){
+			return 1;
+		}else return 0;
+	}else if (secdiff < 0){
+		return -1;
+	}else
+		return 1;
+}
+
+
+uint32_t bctbx_time_string_to_sec(const char *timeString) {
+
+	char *p = NULL;
+	char *o = NULL;
+	int32_t n=0;
+	uint32_t ret=0;
+
+	if (timeString == NULL) {
+		return 0;
+	}
+
+	o = p = bctbx_strdup(timeString);
+
+	while (*p!='\0') {
+		n=strtol(p, &p, 10);
+		switch (*p) {
+			case '\0':
+				ret+=n;
+			break;
+
+			case 'Y':
+				ret +=n*365*24*3600;
+				p++;
+			break;
+
+			case 'M':
+				ret +=n*30*24*3600;
+				p++;
+			break;
+
+			case 'W':
+				ret +=n*7*24*3600;
+				p++;
+			break;
+
+			case 'd':
+				ret +=n*24*3600;
+				p++;
+			break;
+
+			case 'h':
+				ret +=n*3600;
+				p++;
+			break;
+
+			case 'm':
+				ret +=n*60;
+				p++;
+			break;
+
+			case 's':
+				ret+=n;
+				p++;
+			break;
+
+			default: /* just ignore any other suffix */
+				p++;
+			break;
+		}
+	}
+	bctbx_free(o);
+	return ret;
+}
+
 #if defined(_WIN32) && !defined(_MSC_VER)
 char* strtok_r(char *str, const char *delim, char **nextp){
 	char *ret;
@@ -906,13 +1019,6 @@ bool_t bctbx_is_multicast_addr(const struct sockaddr *addr) {
 
 #ifdef _WIN32
 
-int bctbx_bind(bctbx_socket_t socket, const struct sockaddr *address, socklen_t address_len) {
-	return bind(socket, address, (int)address_len);
-}
-
-int bctbx_connect(bctbx_socket_t socket, const struct sockaddr *address, socklen_t address_len) {
-	return connect(socket, address, (int)address_len);
-}
 
 ssize_t bctbx_send(bctbx_socket_t socket, const void *buffer, size_t length, int flags) {
 	return send(socket, (const char *)buffer, (int)length, flags);
@@ -940,14 +1046,6 @@ ssize_t bctbx_write(int fd, const void *buf, size_t nbytes) {
 
 #else
 
-int bctbx_bind(bctbx_socket_t socket, const struct sockaddr *address, socklen_t address_len) {
-	return bind(socket, address, address_len);
-}
-
-int bctbx_connect(bctbx_socket_t socket, const struct sockaddr *address, socklen_t address_len) {
-	return connect(socket, address, address_len);
-}
-
 ssize_t bctbx_send(bctbx_socket_t socket, const void *buffer, size_t length, int flags) {
 	return send(socket, buffer, length, flags);
 }
@@ -974,14 +1072,16 @@ ssize_t bctbx_write(int fd, const void *buf, size_t nbytes) {
 
 #endif
 
+static char allocated_by_bctbx_magic[10] = "bctbx";
 
 static struct addrinfo *_bctbx_alloc_addrinfo(int ai_family, int socktype, int proto){
-	struct addrinfo *ai=(struct addrinfo*)bctbx_malloc0(sizeof(struct addrinfo));
+	struct addrinfo *ai=(struct addrinfo*)bctbx_malloc0(sizeof(struct addrinfo) + sizeof(struct sockaddr_storage));
 	ai->ai_family=ai_family;
 	ai->ai_socktype=socktype;
 	ai->ai_protocol=proto;
 	ai->ai_addrlen=AF_INET6 ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr_in);
-	ai->ai_addr=(struct sockaddr *) bctbx_malloc0(ai->ai_addrlen);
+	ai->ai_addr=(struct sockaddr*)(((unsigned char*)ai) + sizeof(struct addrinfo));
+	ai->ai_canonname = allocated_by_bctbx_magic; /*this is the way we will recognize our own allocated addrinfo structures in bctbx_freeaddrinfo()*/
 	return ai;
 }
 
@@ -1013,7 +1113,7 @@ static struct addrinfo *convert_to_v4mapped(const struct addrinfo *ai){
 	return res;
 }
 
-#if defined(ANDROID) || defined(_WIN32)
+#if defined(__ANDROID__) || defined(_WIN32)
 
 /*
  * SHAME !!! bionic's getaddrinfo does not implement the AI_V4MAPPED flag !
@@ -1049,47 +1149,25 @@ int bctbx_getaddrinfo(const char *node, const char *service, const struct addrin
 		if (hints) memcpy(&lhints,hints,sizeof(lhints));
 		
 		lhints.ai_flags &= ~(AI_ALL | AI_V4MAPPED); /*remove the unsupported flags*/
-		if (hints->ai_flags & AI_ALL){
-			lhints.ai_family=AF_INET6;
-			err=getaddrinfo(node, service, &lhints, &res6);
+		lhints.ai_family = AF_INET6;
+		err = getaddrinfo(node, service, &lhints, &res6);
+		if (hints->ai_flags & AI_ALL) {
+			lhints.ai_family=AF_INET;
+			err=getaddrinfo(node, service, &lhints, &res4);
+			if (err==0){
+				struct addrinfo *v4m=convert_to_v4mapped(res4);
+				freeaddrinfo(res4);
+				res4=v4m;
+			}
+			*res=addrinfo_concat(res6,res4);
+			if (*res) err=0;
 		}
-		lhints.ai_family=AF_INET;
-		err=getaddrinfo(node, service, &lhints, &res4);
-		if (err==0){
-			struct addrinfo *v4m=convert_to_v4mapped(res4);
-			freeaddrinfo(res4);
-			res4=v4m;
-		}
-		*res=addrinfo_concat(res6,res4);
-		if (*res) err=0;
 		return err;
 	}
 	return getaddrinfo(node, service, hints, res);
 }
 
-void _bctbx_freeaddrinfo(struct addrinfo *res){
-	struct addrinfo *it,*next_it;
-	for(it=res;it!=NULL;it=next_it){
-		next_it=it->ai_next;
-		bctbx_free(it->ai_addr);
-		bctbx_free(it);
-	}
-}
 
-void bctbx_freeaddrinfo(struct addrinfo *res){
-	struct addrinfo *it,*previt=NULL;
-	struct addrinfo *allocated_by_bctbx=NULL;
-	for(it=res;it!=NULL;it=it->ai_next){
-		if (it->ai_flags & AI_V4MAPPED){
-			allocated_by_bctbx=it;
-			if (previt) previt->ai_next=NULL;
-			break;
-		}
-		previt=it;
-	}
-	if (res!=allocated_by_bctbx) freeaddrinfo(res);
-	if (allocated_by_bctbx) _bctbx_freeaddrinfo(allocated_by_bctbx);
-}
 
 #else
 
@@ -1113,12 +1191,26 @@ int bctbx_getaddrinfo(const char *node, const char *service, const struct addrin
 
 }
 
-void bctbx_freeaddrinfo(struct addrinfo *res){
-	freeaddrinfo(res);
-}
-
 #endif
 
+int bctbx_getnameinfo(const struct sockaddr *addr, socklen_t addrlen, char *host, socklen_t hostlen, char *serv, socklen_t servlen, int flags) {
+#if __APPLE__
+	/* What an unpleasant surprise... It appears getnameinfo from Apple is calling inet_ntoa internally that is not thread-safe:
+	 *   https://opensource.apple.com/source/Libc/Libc-583/net/FreeBSD/inet_ntoa.c
+	 *   http://www.educatedguesswork.org/2009/02/well_thats_an_unpleasant_surpr.html
+	 */
+	int i;
+	int err;
+	for (i = 0; i < 50; i++) {
+		err = getnameinfo(addr, addrlen, host, hostlen, serv, servlen, flags);
+		if (!(host && strstr(host, "[inet_ntoa error]"))) return err;
+	}
+	return EAI_AGAIN;
+#else
+	return getnameinfo(addr, addrlen, host, hostlen, serv, servlen, flags);
+#endif
+}
+
 static void _bctbx_addrinfo_to_ip_address_error(int err, char *ip, size_t ip_size) {
 	bctbx_error("getnameinfo() error: %s", gai_strerror(err));
 	strncpy(ip, "<bug!!>", ip_size);
@@ -1126,7 +1218,7 @@ static void _bctbx_addrinfo_to_ip_address_error(int err, char *ip, size_t ip_siz
 
 int bctbx_addrinfo_to_ip_address(const struct addrinfo *ai, char *ip, size_t ip_size, int *port){
 	char serv[16];
-	int err=getnameinfo(ai->ai_addr,(socklen_t)ai->ai_addrlen,ip,(socklen_t)ip_size,serv,(socklen_t)sizeof(serv),NI_NUMERICHOST|NI_NUMERICSERV);
+	int err=bctbx_getnameinfo(ai->ai_addr,(socklen_t)ai->ai_addrlen,ip,(socklen_t)ip_size,serv,(socklen_t)sizeof(serv),NI_NUMERICHOST|NI_NUMERICSERV);
 	if (err!=0) _bctbx_addrinfo_to_ip_address_error(err, ip, ip_size);
 	if (port) *port=atoi(serv);
 	return 0;
@@ -1135,7 +1227,7 @@ int bctbx_addrinfo_to_ip_address(const struct addrinfo *ai, char *ip, size_t ip_
 int bctbx_addrinfo_to_printable_ip_address(const struct addrinfo *ai, char *printable_ip, size_t printable_ip_size) {
 	char ip[64];
 	char serv[16];
-	int err = getnameinfo(ai->ai_addr, (socklen_t)ai->ai_addrlen, ip, sizeof(ip), serv, sizeof(serv), NI_NUMERICHOST | NI_NUMERICSERV);
+	int err = bctbx_getnameinfo(ai->ai_addr, (socklen_t)ai->ai_addrlen, ip, sizeof(ip), serv, sizeof(serv), NI_NUMERICHOST | NI_NUMERICSERV);
 	if (err != 0) _bctbx_addrinfo_to_ip_address_error(err, ip, sizeof(ip));
 	if (ai->ai_family == AF_INET)
 		snprintf(printable_ip, printable_ip_size, "%s:%s", ip, serv);
@@ -1144,9 +1236,9 @@ int bctbx_addrinfo_to_printable_ip_address(const struct addrinfo *ai, char *prin
 	return 0;
 }
 
-int bctbx_sockaddr_to_ip_address(struct sockaddr *sa, socklen_t salen, char *ip, size_t ip_size, int *port) {
+int bctbx_sockaddr_to_ip_address(const struct sockaddr *sa, socklen_t salen, char *ip, size_t ip_size, int *port) {
 	struct addrinfo ai = { 0 };
-	ai.ai_addr = sa;
+	ai.ai_addr = (struct sockaddr*)sa;
 	ai.ai_addrlen = salen;
 	ai.ai_family = sa->sa_family;
 	return bctbx_addrinfo_to_ip_address(&ai, ip, ip_size, port);
@@ -1176,8 +1268,9 @@ static struct addrinfo * _bctbx_name_to_addrinfo(int family, int socktype, const
 	if (numeric_only) hints.ai_flags=AI_NUMERICSERV|AI_NUMERICHOST;
 	hints.ai_socktype=socktype;
 	
-	if (family==AF_INET6 && strchr(ipaddress,':')==NULL) {
-		hints.ai_flags|=AI_V4MAPPED;
+	if (family == AF_INET6) {
+		hints.ai_flags |= AI_V4MAPPED;
+		hints.ai_flags |= AI_ALL;
 	}
 	err=bctbx_getaddrinfo(ipaddress,serv,&hints,&res);
 
@@ -1235,14 +1328,15 @@ void bctbx_sockaddr_remove_nat64_mapping(const struct sockaddr *v6, struct socka
 	if (v6->sa_family == AF_INET6) {
 		struct sockaddr_in6 *in6 = (struct sockaddr_in6 *)v6;
 
+		if (htonl(0x0064ff9b) ==
 #ifdef _MSC_VER
-		if ((((in6->sin6_addr.u.Word[0] << 16) & in6->sin6_addr.u.Word[1]) == htonl(0x0064))
-			&& (((in6->sin6_addr.u.Word[2] << 16) & in6->sin6_addr.u.Word[3]) == htonl(0xff9b))) {
+			((in6->sin6_addr.u.Word[0] << 16) & in6->sin6_addr.u.Word[1])
 #elif __APPLE__
-		if ((in6->sin6_addr.__u6_addr.__u6_addr32[0] == htonl(0x0064)) && (in6->sin6_addr.__u6_addr.__u6_addr32[1] == htonl(0xff9b))) {
+			in6->sin6_addr.__u6_addr.__u6_addr32[0]
 #else
-		if ((in6->sin6_addr.s6_addr32[0] == htonl(0x0064)) && (in6->sin6_addr.s6_addr32[1] == htonl(0xff9b))) {
+			in6->sin6_addr.s6_addr32[0]
 #endif
+		) {
 			struct sockaddr_in *in = (struct sockaddr_in *)result;
 			result->sa_family = AF_INET;
 			in->sin_addr.s_addr = IN6_GET_ADDR_V4MAPPED(&in6->sin6_addr);
@@ -1257,9 +1351,6 @@ void bctbx_sockaddr_remove_nat64_mapping(const struct sockaddr *v6, struct socka
 
 void bctbx_sockaddr_ipv6_to_ipv4(const struct sockaddr *v6, struct sockaddr *result, socklen_t *result_len) {
 	bctbx_sockaddr_remove_v4_mapping(v6, result, result_len);
-	if (result->sa_family == AF_INET6) {
-		bctbx_sockaddr_remove_nat64_mapping(v6, result, result_len);
-	}
 }
 
 void bctbx_sockaddr_ipv4_to_ipv6(const struct sockaddr *v4, struct sockaddr *result, socklen_t *result_len) {
@@ -1273,6 +1364,7 @@ void bctbx_sockaddr_ipv4_to_ipv6(const struct sockaddr *v4, struct sockaddr *res
 		v4m = convert_to_v4mapped(&ai);
 		*result_len = sizeof(struct sockaddr_in6);
 		memcpy(v6, v4m->ai_addr, *result_len);
+		bctbx_freeaddrinfo(v4m);
 	}
 }
 
@@ -1387,11 +1479,14 @@ static int get_local_ip_for_with_connect(int type, const char *dest, int port, c
 	}
 	tmp = 1;
 	err = setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (SOCKET_OPTION_VALUE)&tmp, sizeof(int));
-	if (err == -1) bctbx_warning("Error in setsockopt: %s", strerror(errno));
+	if (err == -1) bctbx_warning("Error in setsockopt: %s", getSocketError());
 	err = connect(sock, res->ai_addr, (int)res->ai_addrlen);
 	if (err == -1) {
-		/* The network isn't reachable */
-		if (getSocketErrorCode() != BCTBX_ENETUNREACH) bctbx_error("Error in connect: %s", strerror(errno));
+		/* The network isn't reachable. We don't display the error as it is the case that we want to check in normal operation. */
+		if (   getSocketErrorCode() != BCTBX_ENETUNREACH
+			&& getSocketErrorCode() != BCTBX_EHOSTUNREACH
+			&& getSocketErrorCode() != BCTBX_EPROTOTYPE)
+				bctbx_error("Error in connect: %s", getSocketError());
 		freeaddrinfo(res);
 		bctbx_socket_close(sock);
 		return -1;
@@ -1401,7 +1496,7 @@ static int get_local_ip_for_with_connect(int type, const char *dest, int port, c
 	s = sizeof(addr);
 	err = getsockname(sock, (struct sockaddr *)&addr, &s);
 	if (err != 0) {
-		bctbx_error("Error in getsockname: %s", strerror(errno));
+		bctbx_error("Error in getsockname: %s", getSocketError());
 		bctbx_socket_close(sock);
 		return -1;
 	}
@@ -1412,8 +1507,8 @@ static int get_local_ip_for_with_connect(int type, const char *dest, int port, c
 			return -1;
 		}
 	}
-	err = getnameinfo((struct sockaddr *)&addr, s, result, result_len, NULL, 0, NI_NUMERICHOST);
-	if (err != 0) bctbx_error("getnameinfo error: %s", strerror(errno));
+	err = bctbx_getnameinfo((struct sockaddr *)&addr, s, result, (socklen_t)result_len, NULL, 0, NI_NUMERICHOST);
+	if (err != 0) bctbx_error("getnameinfo error: %s", gai_strerror(err));
 	/* Avoid ipv6 link-local addresses */
 	if ((p_addr->sa_family == AF_INET6) && (strchr(result, '%') != NULL)) {
 		strcpy(result, "::1");
@@ -1425,7 +1520,6 @@ static int get_local_ip_for_with_connect(int type, const char *dest, int port, c
 }
 
 int bctbx_get_local_ip_for(int type, const char *dest, int port, char *result, size_t result_len) {
-	int err;
 	strncpy(result, (type == AF_INET) ? "127.0.0.1" : "::1", result_len);
 
 	if (dest == NULL) {
@@ -1435,3 +1529,150 @@ int bctbx_get_local_ip_for(int type, const char *dest, int port, char *result, s
 	if (port == 0) port = 5060;
 	return get_local_ip_for_with_connect(type, dest, port, result, result_len);
 }
+
+void _bctbx_freeaddrinfo(struct addrinfo *res){
+	struct addrinfo *it,*next_it;
+	for(it=res;it!=NULL;it=next_it){
+		next_it=it->ai_next;
+		bctbx_free(it);
+	}
+}
+
+void bctbx_freeaddrinfo(struct addrinfo *res){
+	struct addrinfo *it;
+	struct addrinfo *previt = NULL;
+	struct addrinfo *beginit = res;
+	bool_t looking_for_allocated_by_bctbx = (res->ai_canonname == allocated_by_bctbx_magic) ? TRUE : FALSE;
+
+	for (it = res; it != NULL; it = it->ai_next) {
+		if ((looking_for_allocated_by_bctbx == TRUE) && (it->ai_canonname != allocated_by_bctbx_magic)) {
+			if (previt) {
+				previt->ai_next = NULL;
+				_bctbx_freeaddrinfo(beginit);
+				looking_for_allocated_by_bctbx = FALSE;
+				beginit = it;
+			}
+		} else if ((looking_for_allocated_by_bctbx == FALSE) && (it->ai_canonname == allocated_by_bctbx_magic)) {
+			if (previt) {
+				previt->ai_next = NULL;
+				freeaddrinfo(beginit);
+				looking_for_allocated_by_bctbx = TRUE;
+				beginit = it;
+			}
+		}
+		previt = it;
+	}
+	if (looking_for_allocated_by_bctbx == TRUE) _bctbx_freeaddrinfo(beginit);
+	else freeaddrinfo(beginit);
+}
+
+
+uint8_t bctbx_char_to_byte(uint8_t input_char) {
+	/* 0-9 */
+	if (input_char>0x29 && input_char<0x3A) {
+		return input_char - 0x30;
+	}
+
+	/* a-f */
+	if (input_char>0x60 && input_char<0x67) {
+		return input_char - 0x57; /* 0x57 = 0x61(a) + 0x0A*/
+	}
+
+	/* A-F */
+	if (input_char>0x40 && input_char<0x47) {
+		return input_char - 0x37; /* 0x37 = 0x41(a) + 0x0A*/
+	}
+
+	/* shall never arrive here, string is not Hex*/
+	return 0;
+
+}
+
+uint8_t bctbx_byte_to_char(uint8_t input_byte) {
+	uint8_t input_byte_crop = input_byte&0x0F; /* restrict the input value to range [0-15] */
+	/* 0-9 */
+	if(input_byte_crop<0x0A) {
+		return input_byte_crop+0x30;
+	}
+	/* a-f */
+	return input_byte_crop + 0x57;
+}
+
+void bctbx_str_to_uint8(uint8_t *output_bytes, const uint8_t *input_string, size_t input_string_length) {
+	size_t i;
+	for (i=0; i<input_string_length/2; i++) {
+		output_bytes[i] = (bctbx_char_to_byte(input_string[2*i]))<<4 | bctbx_char_to_byte(input_string[2*i+1]);
+	}
+}
+
+void bctbx_int8_to_str(uint8_t *output_string, const uint8_t *input_bytes, size_t input_bytes_length) {
+	size_t i;
+	for (i=0; i<input_bytes_length; i++) {
+		output_string[2*i] = bctbx_byte_to_char((input_bytes[i]>>4)&0x0F);
+		output_string[2*i+1] = bctbx_byte_to_char(input_bytes[i]&0x0F);
+	}
+}
+
+void bctbx_uint32_to_str(uint8_t output_string[9], uint32_t input_uint32) {
+
+	output_string[0] = bctbx_byte_to_char((uint8_t)((input_uint32>>28)&0x0F));
+	output_string[1] = bctbx_byte_to_char((uint8_t)((input_uint32>>24)&0x0F));
+	output_string[2] = bctbx_byte_to_char((uint8_t)((input_uint32>>20)&0x0F));
+	output_string[3] = bctbx_byte_to_char((uint8_t)((input_uint32>>16)&0x0F));
+	output_string[4] = bctbx_byte_to_char((uint8_t)((input_uint32>>12)&0x0F));
+	output_string[5] = bctbx_byte_to_char((uint8_t)((input_uint32>>8)&0x0F));
+	output_string[6] = bctbx_byte_to_char((uint8_t)((input_uint32>>4)&0x0F));
+	output_string[7] = bctbx_byte_to_char((uint8_t)((input_uint32)&0x0F));
+	output_string[8] = '\0';
+}
+
+uint32_t bctbx_str_to_uint32(const uint8_t *input_string) {
+	return  (((uint32_t)bctbx_char_to_byte(input_string[0]))<<28)
+		| (((uint32_t)bctbx_char_to_byte(input_string[1]))<<24)
+		| (((uint32_t)bctbx_char_to_byte(input_string[2]))<<20)
+		| (((uint32_t)bctbx_char_to_byte(input_string[3]))<<16)
+		| (((uint32_t)bctbx_char_to_byte(input_string[4]))<<12)
+		| (((uint32_t)bctbx_char_to_byte(input_string[5]))<<8)
+		| (((uint32_t)bctbx_char_to_byte(input_string[6]))<<4)
+		| (((uint32_t)bctbx_char_to_byte(input_string[7])));
+}
+
+void bctbx_uint64_to_str(uint8_t output_string[17], uint64_t input_uint64) {
+
+	output_string[0] = bctbx_byte_to_char((uint8_t)((input_uint64>>60)&0x0F));
+	output_string[1] = bctbx_byte_to_char((uint8_t)((input_uint64>>56)&0x0F));
+	output_string[2] = bctbx_byte_to_char((uint8_t)((input_uint64>>52)&0x0F));
+	output_string[3] = bctbx_byte_to_char((uint8_t)((input_uint64>>48)&0x0F));
+	output_string[4] = bctbx_byte_to_char((uint8_t)((input_uint64>>44)&0x0F));
+	output_string[5] = bctbx_byte_to_char((uint8_t)((input_uint64>>40)&0x0F));
+	output_string[6] = bctbx_byte_to_char((uint8_t)((input_uint64>>36)&0x0F));
+	output_string[7] = bctbx_byte_to_char((uint8_t)((input_uint64>>32)&0x0F));
+	output_string[8] = bctbx_byte_to_char((uint8_t)((input_uint64>>28)&0x0F));
+	output_string[9] = bctbx_byte_to_char((uint8_t)((input_uint64>>24)&0x0F));
+	output_string[10] = bctbx_byte_to_char((uint8_t)((input_uint64>>20)&0x0F));
+	output_string[11] = bctbx_byte_to_char((uint8_t)((input_uint64>>16)&0x0F));
+	output_string[12] = bctbx_byte_to_char((uint8_t)((input_uint64>>12)&0x0F));
+	output_string[13] = bctbx_byte_to_char((uint8_t)((input_uint64>>8)&0x0F));
+	output_string[14] = bctbx_byte_to_char((uint8_t)((input_uint64>>4)&0x0F));
+	output_string[15] = bctbx_byte_to_char((uint8_t)((input_uint64)&0x0F));
+	output_string[16] = '\0';
+}
+
+uint64_t bctbx_str_to_uint64(const uint8_t input_string[17]) {
+	return  (((uint64_t)bctbx_char_to_byte(input_string[0]))<<60)
+		| (((uint64_t)bctbx_char_to_byte(input_string[1]))<<56)
+		| (((uint64_t)bctbx_char_to_byte(input_string[2]))<<52)
+		| (((uint64_t)bctbx_char_to_byte(input_string[3]))<<48)
+		| (((uint64_t)bctbx_char_to_byte(input_string[4]))<<44)
+		| (((uint64_t)bctbx_char_to_byte(input_string[5]))<<40)
+		| (((uint64_t)bctbx_char_to_byte(input_string[6]))<<36)
+		| (((uint64_t)bctbx_char_to_byte(input_string[7]))<<32)
+		| (((uint64_t)bctbx_char_to_byte(input_string[8]))<<28)
+		| (((uint64_t)bctbx_char_to_byte(input_string[9]))<<24)
+		| (((uint64_t)bctbx_char_to_byte(input_string[10]))<<20)
+		| (((uint64_t)bctbx_char_to_byte(input_string[11]))<<16)
+		| (((uint64_t)bctbx_char_to_byte(input_string[12]))<<12)
+		| (((uint64_t)bctbx_char_to_byte(input_string[13]))<<8)
+		| (((uint64_t)bctbx_char_to_byte(input_string[14]))<<4)
+		| (((uint64_t)bctbx_char_to_byte(input_string[15])));
+}
